#ifndef R2_CONSTS_H
#define R2_CONSTS_H

#define KB                   1024
#define MB                   (1024*KB)
#define GB                   (1024*MB)
#define PAGESIZE             (4*KB)

#define X                    0x20000

#define PSR_V (1<<28)
#define PSR_C (1<<29)

#define OS_WriteC            0x00
#define OS_WriteS            0x01
#define OS_Write0            0x02
#define OS_NewLine           0x03
#define OS_ReadC             0x04
#define OS_CLI               0x05
#define OS_Byte              0x06
#define OS_Word              0x07
#define OS_File              0x08
#define OS_Args              0x09
#define OS_BGet              0x0a
#define OS_BPut              0x0b
#define OS_GBPB              0x0c
#define OS_Find              0x0d
#define OS_ReadLine          0x0e
#define OS_GetEnv            0x10
#define OS_Exit              0x11
#define OS_Heap              0x1d
#define OS_Module            0x1e
#define OS_Claim             0x1f
#define OS_ReadVarVal        0x23
#define OS_SetVarVal         0x24
#define OS_GSTrans           0x27
#define OS_FSControl         0x29
#define OS_ServiceCall       0x30
#define OS_ReadVduVariables  0x31
#define OS_ChangeEnvironment 0x40
#define OS_PrettyPrint       0x44
#define OS_WriteN            0x46
#define OS_ReadSysInfo       0x58
#define OS_ReadDynamicArea   0x5c
#define OS_ClaimSWI          0x62
#define OS_SynchroniseCodeAreas 0x6e
#define OS_ConvertCardinal1  0xd5
#define OS_ConvertCardinal2  0xd6
#define OS_ConvertCardinal3  0xd7
#define OS_ConvertCardinal4  0xd8
#define OS_WriteI            0x100

#define R2K_SWIBase              0x00FFFFC0
#define R2K_Init                 (R2K_SWIBase + 0)
#define R2K_Halt                 (R2K_SWIBase + 1)
#define R2K_PutChar              (R2K_SWIBase + 2)
#define R2K_GetChar              (R2K_SWIBase + 3)
#define R2K_FlushCache           (R2K_SWIBase + 4)
#define R2K_AllocatePhysicalPage (R2K_SWIBase + 5)
#define R2K_MapPhysicalPage      (R2K_SWIBase + 6)
#define R2K_AllocateVirtualZone  (R2K_SWIBase + 7)

#define Vector_UserV 0x00
#define Vector_ErrorV 0x01
#define Vector_IrqV 0x02
#define Vector_WrchV 0x03
#define Vector_RdchV 0x04
#define Vector_CliV 0x05
#define Vector_ByteV 0x06
#define Vector_WordV 0x07
#define Vector_FileV 0x08
#define Vector_ArgsV 0x09
#define Vector_BGetV 0x0A
#define Vector_BPutV 0x0B
#define Vector_GBPBV 0x0C
#define Vector_FindV 0x0D
#define Vector_ReadLineV 0x0E
#define Vector_FSCV 0x0F
#define Vector_EventV 0x10
#define Vector_UPTV 0x11
#define Vector_NETV 0x12
#define Vector_KEYV 0x13
#define Vector_INSV 0x14
#define Vector_REMV 0x15
#define Vector_CNPV 0x16
#define Vector_UKVDU23V 0x17
#define Vector_UKSWIV 0x18
#define Vector_UKPLOTV 0x19
#define Vector_MouseV 0x1A
#define Vector_VDUXV 0x1B
#define Vector_TickerV 0x1C
#define Vector_UpCallV 0x1D
#define Vector_ChangeEnvironmentV 0x1E
#define Vector_SpriteV 0x1F
#define Vector_DrawV 0x20
#define Vector_EconetV 0x21
#define Vector_ColourV 0x22
#define Vector_PaletteV 0x23
#define Vector_SerialV 0x24
#define Vector_FontV 0x25
#define Vector_PointerV 0x26
#define Vector_TimeShareV 0x27
#define Vector_GraphicsV 0x2A
#define Vector_UnthreadV 0x2B
#define Vector_SpareVector4 0x2C
#define Vector_SpareVector3 0x2D
#define Vector_SpareVector2 0x2E
#define Vector_SpareVector1 0x2F
#define Vector__Count 0x30

#define Handler_MemoryLimit            0
#define Handler_UndefinedInstruction   1
#define Handler_PrefetchAbort          2
#define Handler_DataAbort              3
#define Handler_AddressException       4
#define Handler_OtherException         5
#define Handler_Error                  6
#define Handler_Callback               7
#define Handler_Breakpoint             8
#define Handler_Escape                 9
#define Handler_Event                  10
#define Handler_Exit                   11
#define Handler_UnusedSWI              12
#define Handler_ExceptionRegisters     13
#define Handler_ApplicationSpace       14
#define Handler_CurrentlyActiveObject  15
#define Handler_Upcall                 16
#define Handler__Count                 17

#define VduExt_WindowWidth             256

/* Indicates a dictionary term in a prettyprint string. */
#define TokenEscapeChar                27

/* System variable types. */

#define VarType_String                 0
#define VarType_Number                 1
#define VarType_Macro                  2
#define VarType_Expanded               3
#define VarType_LiteralString          4
#define VarType_Code                   16

/* Error numbers. */

#define Error_RMNotFound               0x102
#define Error_NoMoreModules            0x10a
#define Error_NoMoreIncarnations       0x10b

#define Error_BadMacVal                0x120
#define Error_BadVarNam                0x121
#define Error_BadVarType               0x122
#define Error_VarNoRoom                0x122
#define Error_VarCantFind              0x122
#define Error_VarTooLong               0x122

#endif
