#ifndef R2K_H
#define R2K_H

#include "r2.h"

extern void* memset(void* dst, int c, size_t length);
extern int memcmp(const void* s1, const void* s2, size_t n);
extern void* memcpy(void* dest, const void* src, size_t n);
extern int strcmp(const char* s1, const char* s2);
extern size_t strlen(const char* s);

extern int putchar(int c);
extern int getchar(void);
extern int printf(const char* format, ...);
extern int puts(const char* s);
extern int itoa(char* buffer, size_t blen, int value);
extern int utoa(char* buffer, size_t blen, int value);

extern void abort(void);

#define assert(c) \
	do { \
		if (!(c)) \
			printf("Assertion failure at %s:%d: %s\r\n", \
					__FILE__, __LINE__, #c); \
	} while(0)

/* The system heap. */

#include "kernel/heap.h"

extern void* malloc(size_t size);
extern void free(void* ptr);

extern Heap SystemHeap;
extern void SystemHeapInit(void);

/* Files embedded in the kernel. */

typedef struct
{
	const char* data;
	const size_t length;
	const char* name;
}
RomFile;

extern const RomFile modules_data[];
extern const RomFile files_data[];

/* Callouts. */

#define SYSCALL(n) \
	quad n(Registers* R)

extern quad AsmProxyNested(Registers* R, quad address);
extern quad AsmProxyNoReturn(Registers* R);

/* Various data structures. */

typedef struct
{
	quad routine;
	quad r12;
	quad buffer;
}
SystemControlHandlerDescriptor;

typedef struct Vector
{
	struct Vector* next;
	quad r12;
	quad routine;
}
VectorDescriptor;

typedef struct
{
	quad routine;
	quad r12;
}
SWIDescriptor;

extern VectorDescriptor* VectorTable[Vector__Count];
extern SystemControlHandlerDescriptor SystemControlHandlers[Handler__Count];
extern SWIDescriptor SWITable[0x100];

#define THROWE(n, s) \
	({ \
		static const Error e = {n, s}; \
		R0 = (quad) &e; \
		return PSR_V; \
	})


#include "kernel/module.h"
#include "kernel/amb.h"
#include "kernel/application.h"

#endif
