#include "r2-asm.h"

/* OS_WriteI+value special case. This is called directly from the SWI dispatch
 * layer, so it needs special handling.
 *
 * On entry:
 *   r0-r9 = user parameters (must be preserved)
 *   r11 = SWI without X bit, bottom 8 bits is char to write
 *   lr = not set up, return directly to exit_from_swi
 */

proc WriteI
	stmfd sp!, {r0, lr}

	and r0, r11, #0xff
	mov r12, #Vector_WrchV
	bl VProxy

	ldmfd sp!, {r0, lr}
	b ExitFromSWI



/* OS_WriteS.
 *
 * Writes out an inline string using Vector_WrchV. The string is in [sp, #20],
 * which must be updated after the write.
 *
 * On entry:
 *    r0-r9 = user parameters (must be preserved)
 */

proc Impl_OS_WriteS
	stmfd sp!, {r0, r1, lr}       // REMEMBER! Adjusts stack by 12
	ldr r1, [sp, #20+12]
0:
	ldrb r0, [r1], #1             // load and update
	teq r0, #0
	movne r12, #Vector_WrchV
	adrne lr, 0b
	bne VProxy

	add r1, r1, #3
	bic r1, r1, #3                // round up to the next quad
	str r1, [sp, #20+12]

	ldmfd sp!, {r0, r1, pc}



/* OS_WriteN.
 *
 * Writes out a string of certain length.
 *
 * On entry:
 *    r0 = pointer to string
 *    r1 = number of bytes to write
 *    r2-r9 = must be preserved
 */

proc Impl_OS_WriteN
	stmfd sp!, {r0-r2, lr}
	mov r2, r0                    // r2 becomes pointer
0:
	teq r1, #0
	ldmeqfd sp!, {r0-r2, pc}      // return if r1 == 0
	sub r1, r1, #1                // decrement count

	ldrb r0, [r2], #1             // load and update
	mov r12, #Vector_WrchV
	adr lr, 0b
	b VProxy



/* OS_Write0.
 *
 * Writes out a 0-terminated string.
 *
 * On entry:
 *    r0 = pointer to string
 *    r1-r9 = must be preserved
 */

proc Impl_OS_Write0
	stmfd sp!, {r0-r1, lr}
	mov r1, r0                    // r1 becomes pointer
0:
	ldrb r0, [r1], #1             // load and update
	teq r0, #0
	ldmeqfd sp!, {r0-r1, pc}      // return if zero byte

	mov r12, #Vector_WrchV
	adr lr, 0b
	b VProxy



/* OS_NewLine
 *
 * Write out a newline using Vector_WrchV.
 *
 * On entry:
 *    r0-r9 = user parameters (must be preserved)
 */

proc Impl_OS_NewLine
	stmfd sp!, {r0, lr}

 	mov r0, #13
 	mov r12, #Vector_WrchV
 	bl VProxy

 	mov r0, #10
 	mov r12, #Vector_WrchV
 	bl VProxy

 	ldmfd sp!, {r0, pc}



