#include "r2-asm.h"

/* Proxy an SWI to the C dispatcher. This takes care of all the C runtime setup
 * and shutdown, and passing registers in and out.
 *
 * On entry:
 *   sp = pointer to r10 field of Registers structure
 *   r0-r9 = user parameters
 *   r11 = SWI without X bit
 *   r12 = routine
 */

proc CProxy
	//sub sp, sp, #7*4
	stmfd sp!, {r0-r9}
	mov r0, sp

	push lr

	adr lr, 0f
	mov pc, r12

0:
	pop lr
	msr CPSR_f, r0

	ldmfd sp!, {r0-r9}
	//add sp, sp, #7*4
	mov pc, lr



/* Proxy any routine to the C dispatcher. This is a wrapper around CProxy
 * that fakes up the appropriate stack frame.
 *
 *   r0-r9 = user parameters
 *   r12 = routine
 */

proc FCProxy
	push lr
	sub sp, sp, #7*4              // space for r10-r15, swi
	bl CProxy
	add sp, sp, #7*4
	pop pc



/* Call a vector.
 *
 * On entry:
 *   r0-r9 = user parameters
 *   r11 = SWI without X bit
 *   r12 = vector number
 *
 * Calls a vector, and then returns to lr.
 */

proc VProxy
	ldr r10, = VectorTable
	ldr r10, [r10, r12, lsl #2]   // address of first node

	push lr                       // the evil vector API wants to be able
	                              // to pop the PC off the stack to bypass
	                              // the rest of the vector

	/* Debug code! */
	teq r10, #0
	moveq r0, r12
	beq EmptyVectorHandler

0:
	teq r10, #0
	ldreq pc, [sp], #4            // pop (and exit) on end of list

	adr lr, 0b
	ldmia r10, {r10, r12, pc}     // load next node and call it, in one smooth
	                              // motion



/* Calls a machine code routine from C, nesting it on the current stack.
 * r13 to r15 in the structure are ignored and will be left unmodified.
 *
 * On entry:
 *   r0 = pointer to Registers structure
 *   r1 = routine to call
 */

proc AsmProxyNested
	stmfd sp!, {r4-r11, lr}       // C expects r4-r11 to be preserved
	push r0                       // push address of structure for use after the call
	push r1                       // push address of routine

	ldmia r0, {r0-r12}            // load state
	adr r14, 0f                   // set up lr with return address
	pop pc                        // call routine
0:
	pop lr                        // reload address of structure
	stmia lr, {r0-r12}            // save registers
	ldmfd sp!, {r4-r11, pc}       // return



/* Calls a machine code routine from C. Does not return
 * The r13 and r14 fields in the Registers structure will be used.
 *
 * On entry:
 *   r0 = pointer to Registers structure
 */

proc AsmProxyNoReturn
	ldmia r0, {r0-r15}



/* Last ditch error handler.
 *
 * This fills in the error handler's buffer with the following data
 * block:
 *
 *   +8... string, 0-terminated, max 252 bytes
 *   +4 errno
 *   +0 PC of offending SWI
 * Calls the registered handler in the System Control Handler table.
 * Hopefully, that should do something useful.
 *
 * On entry:
 *   r0 points to the error block
 *   *all* other registers are discardable!
 */

proc Impl_ErrorV
	ldr r1, = SystemControlHandlers

	ldr r2, [r1, #Handler_Error*12 + 8] // get handler's buffer

	ldr r3, [sp, #20+4]          // pull pc off stack (vproxy stacks 1)
	sub r3, r3, #4               // ...move to SWI itself
	str r3, [r2], #4

	ldr r3, [r0], #4              // copy errno into buffer
	str r3, [r2], #4

	mov r4, #252                  // copy max 252 bytes into buffer
0:
	ldrb r3, [r0], #1
	strb r3, [r2], #1
	teq r3, #0
	beq 1f
	subs r4, r4, #1
	bne 0b
1:

	ldr r0, [r1, #Handler_Error*12 + 4] // Note! r12 in r0, not r12!
	ldr pc, [r1, #Handler_Error*12 + 0]



/* Exit handler.
 *
 * No parameters.
 */

proc Impl_OS_Exit
	ldr r1, = SystemControlHandlers

	ldr r0, [r1, #Handler_Exit*12 + 4]
	ldr pc, [r1, #Handler_Exit*12 + 0]

