#include "r2k.h"
#include "module.h"

Module* ModuleList = NULL;

Module* FindModuleByName(const char* name)
{
	Module* md = ModuleList;
	while (md)
	{
		RO_Module* mo = md->module;
		const char* title = &mo->data[mo->header.title];
		if (strcmp(title, name) == 0)
			return md;
		md = md->next;
	}
	return NULL;
}

Module* FindModuleByAddress(quad address)
{
	Module* md = ModuleList;
	while (md)
	{
		RO_Module* mo = md->module;

		quad start = (quad) &mo->data;
		if ((address >= start) &&
			(address < (start+md->length)))
				return md;

		md = md->next;
	}
	return NULL;
}

Module* FindModuleBySWIBase(quad swi)
{
	Module* md = ModuleList;
	while (md)
	{
		if (md->swibase == swi)
			return md;

		md = md->next;
	}
	return NULL;
}

Module* FindModuleByCount(int count)
{
	Module* md = ModuleList;
	while (count--)
	{
		if (!md)
			return NULL;
		md = md->next;
	}
	return md;
}

static void add_module(Module* md)
{
	md->next = ModuleList;
	md->prev = NULL;
	if (ModuleList)
		ModuleList->prev = md;
	ModuleList = md;
}

static quad fixup_header_field(RO_Module* module, quad field)
{
	if (field == 0)
		return 0;
	return field + (quad) module;
}

static Module* insert_module(RO_Module* ro, size_t length)
{
	Module* rd = malloc(sizeof(Module));
	rd->module = ro;
	rd->userword = 0;
	rd->length = length;
	rd->external = 1;

	rd->start = fixup_header_field(ro, ro->header.start);
	rd->init = fixup_header_field(ro, ro->header.init);
	rd->deinit = fixup_header_field(ro, ro->header.deinit);
	rd->service = fixup_header_field(ro, ro->header.service);
	rd->title = fixup_header_field(ro, ro->header.title);
	rd->help = fixup_header_field(ro, ro->header.help);
	rd->commandtable = fixup_header_field(ro, ro->header.commandtable);
	rd->swibase = ro->header.swibase;
	rd->swi = fixup_header_field(ro, ro->header.swi);
	rd->swidecodetable = fixup_header_field(ro, ro->header.swidecodetable);
	rd->swidecode = fixup_header_field(ro, ro->header.swidecode);
	rd->messagetrans = fixup_header_field(ro, ro->header.messagetrans);

	printf("   init = %x\n", rd->init);
	printf("   swibase = %x\n", rd->swibase);
	printf("   swi = %x\n", rd->swi);

	add_module(rd);
	return rd;
}

static void init_module(Module* rd)
{
	/* FIXME: ro->init may also be a literal ARM instruction */
	if (rd->init)
	{
		Registers r;
		Registers* R = &r;

		R10 = 0; /* FIXME: environment string? */
		R11 = 0; /* All modules in RAM */
		R12 = (quad) &rd->userword;
		AsmProxyNested(R, rd->init);
	}
}

#if 0
void OS_Module_InsertInPlace(RO_Module* ro, L4_Word_t length)
{
	const char* title = &ro->data[ro->header.title];
	printf("Insert module %s at %x+%x\n", title, (quad) ro, length);

	/* FIXME: kill existing modules */
	R4_Module* rd = find_module_by_ptr(ro);
	assert(rd == NULL);

	rd = insert_module(ro, length);
	init_module(rd);
}
#endif

void InitRomModules(void)
{
	const RomFile* p = modules_data;

	while (p->data)
	{
		RO_Module* ro = (RO_Module*) p->data;
		const char* title = &ro->data[ro->header.title];

		printf("Init ROM module %s at %x+%x\n", title, (quad) ro,
				p->length);

		Module* rd = insert_module(ro, p->length);
		init_module(rd);
		p++;
	}
}
