#include "r2k.h"

SYSCALL(Impl_OS_ReadSysInfo)
{
	switch (R0)
	{
		case 1: /* mode */
			R0 = R1 = R2 = 0;
			break;

		case 6: /* kernel vars */
		{
			assert(R1 == 0);
			switch (R2)
			{
				case 16: /* SVCSTK */
					R2 = 0;
					break;

				default:
					printf("Unimplemented OS_ReadSysInfo 6 opcode %d!\n", R2);
			}
			break;
		}

		default:
			printf("Unimplemented OS_ReadSysInfo opcode %d!\n", R0);
			abort();
	}

	return 0;
}

SYSCALL(Impl_OS_ReadDynamicArea)
{
	int max = R0 & 128;

	switch (R0 & 0x7F)
	{
		case 0:
			/* FIXME: totally fake */
			R0 = (quad) &SystemHeap;
			R1 = 0x1000;
			if (max)
				R2 = 0x10000000;
			break;

		default:
			printf("Unimplemented OD_ReadDynamicArea %d!\n", R0);
			abort();
	}
	return 0;
}

SYSCALL(Impl_ChangeEnvironmentV)
{
	SystemControlHandlerDescriptor* sch = &SystemControlHandlers[R0];

	quad oldpc = sch->routine;
	quad oldr12 = sch->r12;
	quad oldbuffer = sch->buffer;

	if (R1)
	{
		printf("OS_ChangeEnvironment #%d -> %x, %x\n", R0, R1, R2);
		sch->routine = R1;
		sch->r12 = R2;
	}

	if (R3)
		sch->buffer = R3;

	R1 = oldpc;
	R2 = oldr12;
	R3 = oldbuffer;
	return 0;
}

SYSCALL(Impl_Handler_Error)
{
	quad buffer = SystemControlHandlers[Handler_Error].buffer;
	struct
	{
		quad pc;
		quad e;
		const char msg[];
	}* eblock = (void*) buffer;

	printf("\n%s (%d) occurred at 0x%x\n", eblock->msg, eblock->e, eblock->pc);
	abort();
	return 0;
}
