#include "r2k.h"

enum
{
	NUM_HANDLES = 16
};

typedef struct FileHandle
{
	const RomFile* des;
	quad offset;
}
FileHandle;

static FileHandle files[NUM_HANDLES];

static int clen(const char* name)
{
	int len = 0;

	while (*name++ >= 32)
		len++;
	return len;
}

static const RomFile* find_file_data(const char* name)
{
	int len = clen(name);

	const RomFile* fd = files_data;
	while (fd->data)
	{
		if ((memcmp(name, fd->name, len) == 0) && (strlen(fd->name) == len))
			return fd;
		fd++;
	}
	return NULL;
}

static int allocate_file_handle(void)
{
	int i = 0;
	for (;;)
	{
		if (files[i].des == NULL)
			return i;
		i++;
		assert(i < NUM_HANDLES);
	}
}

SYSCALL(Impl_FindV)
{
	switch (R0 & 0xF0)
	{
		case 0x00: /* Close file */
		{
			int fd = R1-1;
			assert(fd != -1);
			assert(files[fd].des != NULL);
			files[fd].des = NULL;
			break;
		}

		case 0x40: /* Open file R/O */
		{
			const RomFile* des = find_file_data((char*) R1);
			if (!des)
			{
				printf("OS_Find wants to open '%s'\n", (char*) R1);
				abort();
			}

			int fd = allocate_file_handle();
			files[fd].des = des;
			files[fd].offset = 0;
			R0 = fd+1;
			break;
		}

		default:
			printf("Unhandled OS_Find opcode %x!\n", R0);
			abort();
			break;
	}

	return 0;
}

SYSCALL(Impl_FileV)
{
	printf("OS_File %x\n", R0);
	switch (R0)
	{
		case 5: /* Read catalogue info */
		{
			const RomFile* des = find_file_data((char*) R1);
			if (des)
			{
				R0 = 1; /* File */
				R2 = 0; /* Load address */
				R3 = 0; /* Exec address */
				R4 = des->length;
				R5 = 3; /* RW/ */
			}
			else
				R0 = 0;
			break;
		}

		case 0xFF: /* Load file */
		{
			const RomFile* des = find_file_data((char*) R1);
			if (!des)
			{
				printf("OS_File wants to open '%s'\n", (char*) R1);
				abort();
			}

			assert((R3 & 0xFF000000) == 0);
			printf("Load file %s to %x+%x\n", des->name, R2, des->length);
			memcpy((void*) R2, des->data, des->length);
			break;
		}

		default:
			printf("Unhandled OS_File opcode %x!\n", R0);
			abort();
			break;
	}

	return 0;
}

SYSCALL(Impl_ArgsV)
{
	int fd = R1 - 1;

	switch (R0)
	{
		case 2: /* read extent */
			assert(files[fd].des);
			R2 = files[fd].des->length;
			break;

		default:
			printf("Unhandled OS_Args opcode %x!\n", R0);
			abort();
			break;
	}

	return 0;
}

SYSCALL(Impl_OS_FSControl)
{
	switch (R0)
	{
		case 21: /* Get file handle info */
		{
			R1 = R1; /* FS file handle == FileSwitch file handle */
			R2 = 0; /* no FS bits: FIXME */
			break;
		}

		default:
			printf("Unhandled OS_FSControl opcode %x\n", R0);
			abort();
			break;
	}

	return 0;
}
