#include "r2k.h"
#include "module.h"

SYSCALL(Impl_OS_ServiceCall)
{
	/* For each module, call the service handler (if it has one). If R1 == 0
	 * on exit, halt.
	 */

	Module* rd = ModuleList;
	while (rd && (R1 != 0))
	{
		if (rd->service)
			AsmProxyNested(R, rd->service);

		rd = rd->next;
	}

	return 0;
}

SYSCALL(Impl_OS_Module)
{
	switch (R0)
	{
		case 2: /* enter module application */
		{
			Module* md = FindModuleByName((const char*) R1);
			if (!md)
				THROWE(Error_RMNotFound, "RMNotFound");

			R12 = (quad) &md->userword;
			R14 = 0;
			R15 = md->start;
			AsmProxyNoReturn(R);
		}

		case 6: /* allocate RMA memory */
			R2 = (quad) malloc(R3);
			break;

		case 12: /* find module by number */
		{
			/* FIXME: does not support incarnations. */

			Module* md = FindModuleByCount(R1);
			if (!md)
				THROWE(Error_NoMoreModules, "NoMoreModules");
			if (R2 > 0)
				THROWE(Error_NoMoreIncarnations, "NoMoreIncarnations");

			R1 = R1 + 1; /* next module number */
			R2 = 1; /* next incarnation */
			R3 = (quad) md->module; /* module ptr */
			R4 = (quad) md->userword; /* user word */
			R5 = (quad) "Base"; /* incarnation name */
			return 0;
		}

		default:
			printf("Unimplemented OS_Module opcode %d!\n", R0);
			abort();
	}

	return 0;
}
