#include "r2k.h"
#include "heap.h"

Heap SystemHeap;

extern char __end[1];
extern char __top[1];

static void* claim(Heap* heap, size_t* sizep)
{
	size_t size = *sizep;

	printf("System heap full on allocation of size %x!\r\n", size);

	/* Round up to the nearest number of pages. */

	size = size + PAGESIZE-1;
	size = size & ~(PAGESIZE-1);

	/* Request a segment of virtual memory from the monitor. */

	quad address;
	SWI1i1o(R2K_AllocateVirtualZone, size, address);

	printf("Got virtual zone at %x+%x\r\n", address, size);

	/* Map some physical memory into it. */

	for (int i = 0; i < size; i += PAGESIZE)
	{
		quad physical;
		SWI0i1o(R2K_AllocatePhysicalPage, physical);
		SWI2i0o(R2K_MapPhysicalPage, physical, address + i);
	}

	*sizep = size;
	return (void*) address;
}

static int release(Heap* heap, void* ptr, size_t size)
{
	/* Cannot release the first chunk. */

	if (ptr == __end)
		return 0;

	return 0;
}

void* malloc(size_t length)
{
	void* p = HeapAlloc(&SystemHeap, length);
	printf("alloc %x bytes -> %x\n", length, (quad) p);
	return p;
}

void free(void* ptr)
{
	return HeapFree(&SystemHeap, ptr);
}

void SystemHeapInit(void)
{
	static const HeapFunctions fns =
	{
		.claim = claim,
		.release = release
	};

	HeapInit(&SystemHeap, &fns);

	quad length = __top - __end;
	HeapExtend(&SystemHeap, __end, length);

	printf("Initial system heap of %x bytes\r\n", length);
}
