#include <r2k.h>
#include <stdarg.h>

static int digit(int c)
{
	if (c < 10)
		return '0' + c;
	return 'a' + c - 10;
}

int utoa(char* buffer, size_t blen, int value)
{
	char s[10];

	char* p = s;
	do
		*p++ = digit(value % 10);
	while (value /= 10);

	int count = p - s;
	while ((--p >= s) && (blen > 0))
	{
		*buffer++ = *p;
		blen--;
	}

	*buffer = '\0';
	return count;
}

int itoa(char* buffer, size_t blen, int value)
{
	if (blen == 0)
		return 0;

	if (value < 0)
	{
		*buffer = '-';
		return utoa(buffer+1, blen-1, -value) + 1;
	}

	return utoa(buffer, blen, value);
}

int printf(const char* format, ...)
{
	va_list ap;
	va_start(ap, format);

	for (;;)
	{
		int c = *format++;
		switch (c)
		{
			case 0:
				return 0;

			case '%':
			{
				c = *format++;

				switch (c)
				{
					default:
						putchar('%');
						break;

					case 's':
					{
						const char* s = va_arg(ap, const char*);

						for (;;)
						{
							c = *s++;
							if (c == '\0')
								break;
							putchar(c);
						}
						break;
					}

					case 'x':
					{
						quad i = va_arg(ap, quad);

						putchar('0');
						putchar('x');

						for (int j=0; j<8; j++)
						{
							quad d = i >> 28;
							i <<= 4;

							putchar(digit(d));
						}
						break;
					}

					case 'd':
					case 'u':
					{
						quad i = va_arg(ap, quad);

						char buffer[10];
						int count = itoa(buffer, sizeof buffer, i);

						for (int i = 0; i < count; i++)
							putchar(buffer[i]);

						break;
					}

					case 'c':
						break;
				}
				break;
			}

			default:
				putchar(c);
		}
	}
}

