#include "r2.h"
#include "libmodule.h"
#include "globals.h"

static int comparecmd(const char** cmdtablep, const char* pattern)
{
	const char* cmdtable = *cmdtablep;
	int result;

	for (;;)
	{
		int c1 = *cmdtable++;
		int c2 = *pattern++;

		result = ((c1 <= 32) && (c2 <= 32));
		if (result)
			break;
		if (c1 == c2)
			continue;
		break;
	}

	*cmdtablep = (const char*) ((((quad)cmdtable) + 3) & ~3);
	return result;
}

void findCommand(const char* name, CommandBlock** cbp, RO_Module** mp)
{
	/* FIXME. This is a really, really inefficient way of finding commands
	 * (I think it's O(n^2)!). We need to add a command hash to cache them.
	 */

	int mn = 0;
	for (;;)
	{
		int junk;

		/* Find the next module on the chain. */

		RO_Module* module;
		Error* e = SWI3i6o(X|OS_Module, 12, mn, 0,
				junk, mn, junk, module, junk, junk);
		if (e)
			goto error;

		/* Check that this module has a command table. */

		if (!module->header.commandtable)
			continue;
		const char* cmdtable = (const char*)
			&module->data[module->header.commandtable];

		/* Scan it for our command. */

		while (*cmdtable)
		{
			if (comparecmd(&cmdtable, name))
			{
				*cbp = (const CommandBlock*) cmdtable;
				*mp = module;
				return;
			}
			cmdtable += sizeof(CommandBlock);
		}
	}

error:
	*cbp = NULL;
	*mp = NULL;
}
