#include "r2.h"
#include "libmodule.h"
#include "globals.h"

MODULETITLE("R2Variables");
MODULEHELP("R2 System Variable Storage\t12 October 2008");

ENTRYPOINT(ReadVarVal, readvarval_swi);
int readvarval_swi(Registers* R)
{
	/* No data at all? */

	if (!variableListHead)
		goto notfound;

	/* If no start position given, start at the beginning. */

	variable_t* var = variableListHead;
	if (R3)
	{
		var = (variable_t*) (R3 - offsetof(variable_t, name));
		var = var->next;
	}

	/* Look for the variable. */

	var = findVariable(var, (const char*) R0);
	if (!var)
		goto notfound;

	/* First, check to see if there's any data at all. */

	SWI1i0o(X|OS_Write0, "Var fetch not implemented!\n");
	THROWE(0, "Var fetch not implemented!");

	R3 = (quad) &var->name;
	R4 = var->type;
	return 0;

notfound:
	R2 = 0;
	THROWE(Error_VarCantFind, "VarCantFind");
}

/* Set a system variable.
 *
 * => R0 = pointer to name
 * R1 = pointer to value to set to
 * R2 = length, or -1 to delete
 * R3 = name pointer (0 for 1st call)
 * R4 = variable type
 *
 * <= R3 = new name pointer
 * R4 = variable type
 */

ENTRYPOINT(SetVarVal, setvarval_swi);
int setvarval_swi(Registers* R)
{
	/* Is there any data? */

	if (variableListHead)
	{
		/* If so, we may be being asked to delete or replace an existing
		 * variable, so look for it.
		 */

		/* If no start position given, start at the beginning. */

		variable_t* var = variableListHead;
		if (R3)
		{
			var = (variable_t*) (R3 - offsetof(variable_t, name));
			var = var->next;
		}

		/* Look for this variable. */

		var = findVariable(var, (const char*) R0);

		/* If found, remove. */

		if (var)
		{
			/* FIXME: check the type matches. */

			SWI1i0o(X|OS_Write0, "Var delete not implemented!\n");
			THROWE(0, "Var delete not implemented!");
		}
	}

	/* If we've been asked to set a variable, do so. */

	if (R2 != -1)
	{
		variable_t* var = createVariable((const char*) R0, R4, (void*) R1, R2);
		if (!var)
			THROWE(Error_VarNoRoom, "VarNoRoom");

		R3 = (quad) &var->name;
		return 0;
	}

	/* Exit without doing anything more. */

	R3 = 0;
	return 0;
}

#define elementsof(a) (sizeof(a) / sizeof(*a))

ENTRYPOINT(ModuleInit, init);
int init(void* R)
{
	SWI3i0o(X|OS_ClaimSWI, OS_ReadVarVal, ReadVarVal, ModuleUserWord);
	SWI3i0o(X|OS_ClaimSWI, OS_SetVarVal, SetVarVal, ModuleUserWord);
	return 0;
}

ENTRYPOINT(ModuleDeinit, deinit);
int deinit(void* R)
{
	return 0;
}
