#include "r2.h"
#include "libmodule.h"
#include "globals.h"

variable_t* variableListHead;

static int compareWithWildcard(const char* target, const char* pattern)
{
	for (;;)
	{
		int ct = *target;
		int cp = *pattern;

		if ((ct < 32) && (cp < 32))
			return 1;

		if (ct != cp)
			return 0;

		target++;
		pattern++;
	}
}

static size_t varNameLen(const char* s)
{
	size_t n = 0;
	for (;;)
	{
		int c = *s++;
		if (c < 32)
			return n;
		n++;
	}
}

static void varDataCopy(char* dest, const char* src, size_t length)
{
	while (length--)
		*dest++ = *src++;
}

variable_t* findVariable(variable_t* var, const char* pattern)
{
	do
	{
		if (compareWithWildcard(var->name, pattern))
			return var;
		var = var->next;
	}
	while (var);
	return NULL;
}

variable_t* createVariable(const char* name, int type,
		const void* data, size_t datalen)
{
	size_t namelen = varNameLen(name);

	SWI1i0o(X|OS_Write0, "Create var '");
	SWI2i0o(X|OS_WriteN, name, namelen);
	SWI1i0o(X|OS_Write0, "'\n");

	/* Create the variable structure. */

	variable_t* var = ModuleAlloc(sizeof(*var) + namelen + 1);
	if (!var)
		goto error;

	var->type = type;
	varDataCopy(var->name, name, namelen);
	var->name[namelen] = '\0';

	/* Allocate space for the data. */

	var->data = ModuleAlloc(datalen);
	var->datalen = datalen;
	if (!var->data)
		goto error;
	varDataCopy(var->data, data, datalen);

	/* Add this variable to the head of the list. */

	var->next = variableListHead;
	var->prev = NULL;

	if (variableListHead)
		variableListHead->prev = var;

	variableListHead = var;
	return var;

error:
	if (var)
		ModuleFree(var);
	return NULL;
}
