#ifndef LIBMODULE_MODULE_H
#define LIBMODULE_MODULE_H

#include "swimacros.h"

#define SECTION(_N) \
	__attribute__ ((section(_N)))

#define __TOSTRING(x) #x
#define __STRINGIFY(x) __TOSTRING(x)
#define UNIQUESECTION(_C) \
	SECTION(_C "." __FILE__ "." __STRINGIFY(__LINE__))

#define MODULETITLE(s) \
	const char ModuleTitle[] SECTION(".textdata.ModuleTitle") = s

#define MODULEHELP(s) \
	const char ModuleHelp[] SECTION(".textdata.ModuleHelp") = s

#define ENTRYPOINT(name, cfunc) \
	extern void name (void); \
	__asm__ ( \
		".section .text." #name "\n" \
		".globl " #name "\n" \
		#name ":\n" \
			"stmfd sp!, {r0-r12, lr}\n" \
			"mov r0, sp\n" \
			"ldr r10, [r12]\n" \
			"bl " #cfunc "\n" \
			"msr CPSR_f, r0\n" \
			"ldmfd sp!, {r0-r12, pc}\n" \
	)

#define THROW(e) \
	({ \
		R0 = (quad) e; \
		return PSR_V; \
	})

#define THROWE(n, s) \
	({ \
		static const Error e UNIQUESECTION(".text.error") = {n, s}; \
		R0 = (quad) &e; \
		return PSR_V; \
	})

extern void CallAsm(Registers* R);
extern void* ModuleAlloc(size_t len);
extern void ModuleFree(void* ptr);

extern quad ModuleUserWord;

#endif
