#include "r2-asm.h"

proc ModuleXInit
	msr CPSR_f, #0
	msr CPSR_f, r0
	teq r15, #0

	/* Skip the entire setup code if the userdata's already been set up. */

	ldr r2, [r12]
	teq r2, #0
	bne ModuleInit

	/* Allocate the userdata section. */

	mov r0, #6
	ldr r3, = ModuleUserDataSize
	swi X|OS_Module
	bvs abort
	str r2, [r12]                 // r2 = address of userdata

	/* Determine the module load address. */

	adr r1, 0f
0:
	ldr r0, = 0b
	sub r1, r1, r0                // r1 = module load address

	/* Copy the entire data section into the userdata block. */

	ldr r3, = ModuleGotStart      // r3 = offset of GOT in module, read ptr
	mov r5, r2                    // r6 = write pointer
0:
	ldr r0, [r1, r3]
	ldr r4, = ModuleDataStart
	cmp r3, r4                    // GOT table entry?
	bge 1f                        // ...no? Skip further processing, just copy.

	ldr r4, = ModuleGotStart
	cmp r0, r4                    // pointer to something in the module?
	addlt r0, r1                  // ...if so, add on the load address.
	addge r0, r0, r2              // ...add on the GOT table address.
	subge r0, r0, r4              // ...adjust for position in module.
1:
	str r0, [r5], #4              // write value

	add r3, r3, #4
	ldr r4, = ModuleBssStart      // reached end of data?
	teq r3, r4
	bne 0b

	/* r5 is now pointing at the start of the BSS in the module's userdata.
	 * We now want to zero-initialise it. */

	ldr r3, = ModuleBssSize
	mov r0, #0
0:
	teq r3, #0
	beq 1f
	str r0, [r5], #4
	sub r3, r3, #4
	b 0b
1:

	/* Store the userword itself in a global. */

	ldr r0, moduleuserword_got
	ldr r0, [r2, r0]
	str r12, [r0]

	/* Chain call the C routine. */

	b ModuleInit

moduleuserword_got:
	.word ModuleUserWord(GOT)

abort:
	pop pc



proc ModuleXDeinit
	push lr

	/* Call the C routine. */

	bl ModuleDeinit

	/* Free the userdata section. */

	mov r0, #7
	ldr r2, [r12]
	swi X|OS_Module

	/* Zero the pointer, so that it'll be recreated next time. */

	mov r2, #0
	str r2, [r12]

	pop pc

.globl ModuleUserWord
.comm ModuleUserWord, 4
