#include "globals.h"
#include "sleevecc/dis.h"

arm_context context;

void loadfile(const char* filename, arm_byte* address)
{
	int fd = open(filename, O_RDONLY);
	if (fd == -1)
		throw errno;

	for (;;)
	{
		ssize_t s = read(fd, address, 4096);
		if (s == 0)
			break;
		if (s == -1)
		{
			int e = errno;
			close(fd);
			throw e;
		}

		address += s;
	}

	close(fd);
}

int main(int argc, const char* argv[])
{
	try
	{
		sleeve_init();
		vmalloc_init();
		mm_init(mm_TRANSLATION_PAGED, 0xf0000000, 10, 0);

		/* Map 0x00000000-0x00003fff --- RISC OS assumes that 0x00000108
		 * contains important information and will try to read from it. Yuck.
		 */

		mm_map(0x00000000, malloc(0x4000), 0x4000);

		/* Everything from 0x10100000 to 0xf0000000 is available for heap
		 * usage.
		 */

		vmalloc_release(0x10100000, 0xf0000000 - 0x10100000);

		/* We're going to assume that the allocated space is linear. */
		arm_byte* m = mm_map(0x10000000, malloc(1*MB), 1*MB);
		loadfile("bin/kernel.img", m);

		context.r[15] = 0x10000000;

		printf("starting execution\n");
		debugloop();
		arm_state s = arm_execute(&context, 100);
		printf("s=%d\n", s);
	}
	catch (int e)
	{
		printf("Failed: %s\n", strerror(e));
	}

	return 0;
}
