#include "globals.h"
#include "r2k.h"

static quad stackstart;
static quad stackend;

static void r2k_swi(arm_word swi)
{
	switch (swi)
	{
		case R2K_Init:
		{
			stackstart = context.r[0];
			stackend = context.r[1];
			printf("R2 system stack from %08x to %08x\n", stackstart, stackend);
			break;
		}

		case R2K_Halt:
			printf("Halted.\n");
			exit(0);

		case R2K_PutChar:
			putchar(context.r[0]);
			fflush(stdout);
			break;

		case R2K_GetChar:
			context.r[0] = getchar();
			break;

		case R2K_FlushCache:
			break;

		case R2K_AllocatePhysicalPage:
			context.r[0] = (quad) malloc(PAGESIZE);
			//printf("alloc page -> %08x\n", context.r[0]);
			break;

		case R2K_MapPhysicalPage:
			mm_map(context.r[1], (void*) context.r[0], PAGESIZE);
			//printf("map page %08x -> %08x\n", context.r[0], context.r[1]);
			break;

		case R2K_AllocateVirtualZone:
			context.r[0] = vmalloc_claim(context.r[0]);
			break;

		default:
			printf("Unknown R2K SWI %08x!\n", swi);
	}
}

static void user_swi(arm_word swi)
{
	quad sp = context.r[13];
	if ((sp < stackstart) || (sp > stackend))
	{
		//printf("Switching to system stack\n");
		context.r[13] = stackend;
	}

	/* See swi.S for the format of the stack frame. */

	context.r[13] -= 28;
	quad fp = context.r[13];
	mm_store32(fp+12, sp);
	mm_store32(fp+20, context.r[15]);
	mm_store32(fp+24, swi);
	context.r[15] = 0x10000004;
}

int singlestep()
{
	int s = arm_execute(&context, 1);

	switch (s)
	{
		case arm_STATE_SUCCESS:
			return s;

		case arm_STATE_SWI:
		{
			arm_word swi = context.r[arm_SWI];
			//printf("[%08x]", swi);
			if ((swi & ~0x3f) == R2K_SWIBase)
				r2k_swi(swi);
			else
				user_swi(swi);
			return 0;
		}

		default:
			printf("Halted due to %d!\n", s);
			return s;
	}
}

