#include "globals.h"
#include "avl.h"

struct FreeBlock
{
	quad address;
	quad length;

	FreeBlock* nextbysize;
};

static struct FreeBlock* bysize;

void vmalloc_init()
{
	bysize = NULL;
}

static void insert_block(FreeBlock* newblock)
{
	FreeBlock* previous = NULL;
	FreeBlock* fb = bysize;
	while (fb)
	{
		if (fb->length >= newblock->length)
		{
			/* Insert the new block before this one. */

			if (previous)
				previous->nextbysize = newblock;
			else
				bysize = newblock;
			newblock->nextbysize = fb;
			return;
		}

		previous = fb;
		fb = fb->nextbysize;
	}

	/* If we get here, the list is empty and previous is pointing to the
	 * last item. */

	if (previous)
		previous->nextbysize = newblock;
	else
		bysize = newblock;
}

void vmalloc_release(quad address, quad length)
{
	FreeBlock* newblock = new FreeBlock();
	newblock->address = address;
	newblock->length = length;
	newblock->nextbysize = NULL;

	insert_block(newblock);
}

quad vmalloc_claim(quad length)
{
	FreeBlock* previous = NULL;
	FreeBlock* fb = bysize;
	while (fb)
	{
		if (fb->length >= length)
		{
			/* Remove the block from the list. */

			if (previous)
				previous->nextbysize = fb->nextbysize;
			else
				bysize = fb->nextbysize;

			/* Shrink the block. */

			quad address = fb->address;
			fb->address += length;
			fb->length -= length;

			/* If the block is not empty, reinsert it into the right place.
			 * Otherwise delete it. */

			if (fb->length > 0)
				insert_block(fb);
			else
				delete fb;

			return address;
		}
	}

	/* If we got here, we've run out of virtual memory. */

	return 0;
}
